#ifndef _SPIFLASH_H
#define _SPIFLASH_H

#define FLASH_BUFFER_SIZE 528

// Tool for conversion between floats and bytes
typedef union floatbytes {
	float fval;
	char cval[4];
} floatbytes_t;

// Initialize the microcontroller as a master.
void SPI_MasterInit(void);

// Initialize a session of reading contiguous values from Flash
// memory, starting at page page_num and byte 0.
void flash_read_init(int page_num);

// Close a session of reading contiguous values from Flash
void flash_read_complete(void);

// Read next 4 bytes from flash memory and convert to a single float number
// May only be called after flash_read_init has been called
float flash_read_next_float(void);

// Retrieve the status register from Flash
char read_flash_SR(void);

// Writes the first byte_ct bytes in data_buf to the page page_num
// at byte 0 in Flash memory. Erases the page entirely before writing
// new values.
// LEDs also flash on port D to show that a write op is occurring.
// byte_ct MUST only range from 0-528!
void write_flash_page(char* data_buf, int byte_ct, int page_num);

#endif
