function generate_flash_send_code(net)

page_size_bytes = 528;
page_start_hidden = 8;
byte_start_hidden = 0;

[weights_per_hidden hidden_ct] = size(net.hidden_layer_weights);
[weights_per_output output_ct] = size(net.output_layer_weights);

hidden_byte_size = hidden_ct*weights_per_hidden*4;
output_byte_size = output_ct*weights_per_output*4;

page_start_output = page_start_hidden + ceil(hidden_byte_size/page_size_bytes);
byte_start_output = 0;

% some includes and defines

fprintf('// For constants.h:\n');
fprintf('#define PAGE_START_HID %d\n', page_start_hidden);
fprintf('#define PAGE_START_OUT %d\n', page_start_output);
fprintf('#define HIDDEN_NODE_CT %d\n', hidden_ct);
fprintf('#define OUTPUT_NODE_CT %d\n', output_ct);
fprintf('#define X_SCALED %d\n', net.x_size);
fprintf('#define Y_SCALED %d\n', net.y_size);


fprintf('\n');

fprintf('#include <avr/io.h>\n');
fprintf('#include <stdio.h>\n');
fprintf('#include <util/delay.h>\n');
fprintf('#include "constants.h"\n');
fprintf('#include "spiflash.h"\n');
fprintf('#include "uart.h"\n');
fprintf('\n');

fprintf('\n');

fprintf('// UART file descriptor\n');
fprintf('// putchar and getchar are in uart.c\n');
fprintf('FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);\n');
fprintf('\n');

fprintf('void initialize(void) {\n');
fprintf('\tuart_init();\n');
fprintf('\tstdout = stdin = stderr = &uart_str;\n');
fprintf('\tSPI_MasterInit();\n');
fprintf('}\n\n');

fprintf('void forward_serial_to_flash(void) {\n');
fprintf('\tchar serial_buffer[FLASH_BUFFER_SIZE];\n');
fprintf('\tunsigned long expected_bytes_hid = 4*HIDDEN_NODE_CT*(X_SCALED*Y_SCALED+1);\n');
fprintf('\tunsigned long expected_bytes_outp = 4*OUTPUT_NODE_CT*(HIDDEN_NODE_CT+1);\n');
fprintf('\tint page = PAGE_START_HID;\n');
fprintf('\t// PAGE_START_OUT should be %d\n', page_start_output);
fprintf('\tint bytes_recvd = 0;\n');
fprintf('\tlong i;\n');
fprintf('\t\n');
fprintf('\t// Hidden layer weights bytes come first.\n');
fprintf('\tfor(i=0; i<expected_bytes_hid; i++) {\n');
fprintf('\t\tint r = fscanf(stdin, "%%c", &(serial_buffer[bytes_recvd]));\n');
fprintf('\t\tif (r>0) {\n');
fprintf('\t\t\tbytes_recvd++;\n');
fprintf('\t\t}\n');
fprintf('\t\tif (bytes_recvd == FLASH_BUFFER_SIZE) {\n');
fprintf('\t\t\tbytes_recvd = 0;\n');
fprintf('\t\t\twrite_flash_page(serial_buffer, FLASH_BUFFER_SIZE, page++);\n');
fprintf('\t\t}\n');
fprintf('\t}\n');
fprintf('\t// Send last incomplete page of bytes received\n');
fprintf('\tif(bytes_recvd > 0) {\n');
fprintf('\t\twrite_flash_page(serial_buffer, bytes_recvd, page++);\n');
fprintf('\t\tbytes_recvd = 0;\n');
fprintf('\t}\n');
fprintf('\t\n');
fprintf('\t// Output layer weights bytes come second.\n');
fprintf('\tfor(i=0; i<expected_bytes_outp; i++) {\n');
fprintf('\t\tint r = fscanf(stdin, "%%c", &(serial_buffer[bytes_recvd]));\n');
fprintf('\t\tif (r>0) {\n');
fprintf('\t\t\tbytes_recvd++;\n');
fprintf('\t\t}\n');
fprintf('\t\tif (bytes_recvd == FLASH_BUFFER_SIZE) {\n');
fprintf('\t\t\tbytes_recvd = 0;\n');
fprintf('\t\t\twrite_flash_page(serial_buffer, FLASH_BUFFER_SIZE, page++);\n');
fprintf('\t\t}\n');
fprintf('\t}\n');
fprintf('\t// Send last incomplete page of bytes received\n');
fprintf('\tif(bytes_recvd > 0) {\n');
fprintf('\t\twrite_flash_page(serial_buffer, bytes_recvd, page++);\n');
fprintf('\t}\n');
fprintf('}\n\n');

fprintf('int main(void) {\n');
fprintf('\tinitialize();\n');
fprintf('\tDDRD |= 0b00110000;\n');
fprintf('\tPORTD &= ~((1<<5)|(1<<4));\n');
fprintf('\tforward_serial_to_flash()\n');
fprintf('\tPORTD |= (1<<5);\n');
fprintf('\tPORTD &= ~(1<<4);\n');
fprintf('\twhile(1);\n');
fprintf('}\n\n');

end