//Justin Churchill (jec324), Stephen Wu (shw46)
//ECE 4760 Lab 5
//Alphabet Acquire -- Collect entire alphabets of data and send to matlab
//Paired with MATLAB program "getpic.m"

//Reads coordinates from the touchpad to the serial port.

// HARDWARE SETUP:
// PIN A0 : measures x coord. Goes to y1 on board.
// PIN A1 : measures y coord. Goes to x2 on board.
// PIN A2 : needs to be output to set 0 or 5 V. Goes to y2 on board.
// PIN A3 : needs to be output to set 0 or 5 V. Goes to x1 on board.
// 10k resistor between A0 and A1.

// Measuring x: x2(A1) low, x1(A3) high. Sample y1(A0).
// Measuring y: y1(A0) low, y2(A2) high. Sample x2(A1).

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <math.h>
#include "uart.h"
#include "lcd_lib.h"

#define true 1
#define false 0

#define ADC_VREF_5 (1<<REFS0)
#define ADC_VREF_SET(x) ADMUX = (ADMUX & ~(0b11000000)) | x

#define ADC0 0
#define ADC1 1
#define ADC_PIN_SEL(x) ADMUX = (ADMUX & ~(0b00011111)) | x

#define TOUCHPAD_SAMPLE_PERIOD 1
#define PAUSE_TIME 500

#define DEBUG(X) fprintf(stdout, "%d\n", (X))
#define X_MIN_1024 190
#define X_MAX_1024 940 
#define Y_MIN_1024 230
#define Y_MAX_1024 920
#define X_RES 100 // sample resolution in pixels
#define Y_RES 80 // MUST BE MULTIPLES OF 8


// Set up serial for debugging
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

typedef enum coord_t {
	COORD_X,
	COORD_Y
} coord_t;

// Prototypes
void initialize(void);
uint16_t read_touch_pad(coord_t);
void set_PORTA_for_coord(coord_t);
int16_t sample2pt(coord_t, uint16_t);
void print_drawing(uint8_t[][Y_RES/8]);
void clear_drawing(uint8_t[][Y_RES/8]);
void print_norm_drawing(uint8_t[][(Y_SCALED+7)/8]);
void clear_norm_drawing(uint8_t[][(Y_SCALED+7)/8]);
void set_matrix(uint8_t[][Y_RES/8], uint8_t, uint8_t, uint8_t);
uint8_t get_matrix(uint8_t[][Y_RES/8], uint8_t, uint8_t);
void set_norm_matrix(uint8_t[][(Y_SCALED+7)/8], uint8_t, uint8_t, uint8_t);
uint8_t get_norm_matrix(uint8_t[][(Y_SCALED+7)/8], uint8_t, uint8_t);

// Global vars
uint16_t sample_time = TOUCHPAD_SAMPLE_PERIOD;
volatile uint8_t ADC_get_sample = false;
volatile uint8_t sample_on = false;
uint8_t drawing[X_RES][Y_RES/8];

//////////////////////////////////////////////////////////////////////////

int main(void) {
	int i;
	initialize();

	sample_on = true;
	uint16_t counter = 0;
	uint8_t started = false;
	for(;;) {
		if(ADC_get_sample) {
			uint16_t x_samp, y_samp;
			int16_t x_conv, y_conv;
			ADC_get_sample = false;
			x_samp = 1024 - read_touch_pad(COORD_X); // it takes the sample or it gets the hose again
			y_samp = 1024 - read_touch_pad(COORD_Y); // IT TAKES THE SAMPLE OR IT GETS THE HOSE AGAIN
			x_conv= sample2pt(COORD_X, x_samp);
			y_conv= sample2pt(COORD_Y, y_samp);
			//fprintf(stdout, "Raw Data  (%d , %d)\n\r", x_samp, y_samp);
			//fprintf(stdout, "Converted (%d , %d)\n\r", x_conv, y_conv);
			
			if (x_conv >= 0 && y_conv >= 0) {
				set_matrix(drawing, x_conv, y_conv, 1);
				started = true;
				counter = 0;
			}

			if (started == true && (x_conv == -1 || y_conv == -1)) {
				counter++;
				if (counter == PAUSE_TIME) {
					fprintf(stdout, "N\n");
					print_drawing(drawing);
					fprintf(stdout, "E\n");
                    
					clear_drawing(drawing);

					started= false;
				}
			}
		}
	}
	return 0;
}

// Timer 0 compare ISR
ISR (TIMER0_COMPA_vect)
{      
	// Grab a sample from the touch pad every 30 ms
	if (sample_time > 0) {
		sample_time--;
	}
	else {
		sample_time = TOUCHPAD_SAMPLE_PERIOD;
		ADC_get_sample = true;
	}
}

// print the drawing
void print_drawing(uint8_t drawing[][Y_RES/8]) {
	int i, j;
	for (i= 0; i < Y_RES; i++) {
		for (j= 0; j < X_RES; j++) {
			//fprintf(stdout, "%c", (get_matrix(drawing,j,i) == 1 ? '0' : ' '));
			fprintf(stdout, "%c", (get_matrix(drawing,j,i) == 1 ? '1' : '0'));
			//fprintf(stdout, "%d", get_matrix(drawing,j,i));
		}
		fprintf(stdout, "\n");

	}
}

// clear the drawing
void clear_drawing(uint8_t drawing[][Y_RES/8]) {
	int i, j;
	for (i= 0; i < Y_RES; i++) {
		for (j= 0; j < X_RES; j++) {
			set_matrix(drawing, j,i,0);
		}
	}
}

// converts a 10-bit ADC sample to a value between 0 and X_RESOLUTION or Y_RESOLUTION
// returns -1 if the stylus is not touching the screen
int16_t sample2pt(coord_t coord, uint16_t sample) {
	double norm;
	if (coord == COORD_X) {
		norm = (double)sample - X_MIN_1024;
		return (sample < X_MIN_1024 || sample > X_MAX_1024 ? -1 : (int16_t)(norm / (X_MAX_1024 - X_MIN_1024) * X_RES));
	}
	else if (coord == COORD_Y) {
		norm = (double)sample - Y_MIN_1024;
		return (sample < Y_MIN_1024 || sample > Y_MAX_1024 ? -1 : (int16_t)(norm / (Y_MAX_1024 - Y_MIN_1024) * Y_RES));
	}
	else
		return -2; //shouldn't get this
}

// set value at (i,j) in drawing
void set_matrix(uint8_t drawing[][Y_RES/8], uint8_t x, uint8_t y, uint8_t val) {
	drawing[x][y/8] = (val ? drawing[x][y/8] | (1 << (y%8)) : drawing[x][y/8] & ~(1 << (y%8)));
}

// get value at (i,j) in drawing
uint8_t get_matrix(uint8_t drawing[][Y_RES/8], uint8_t x, uint8_t y) {
	return !!(drawing[x][y/8] & (1 << (y%8)));
}

// clear the drawing
void clear_norm_drawing(uint8_t drawing[X_SCALED][(Y_SCALED+7)/8]) {
	int i, j;
	for (i= 0; i < Y_SCALED; i++) {
		for (j= 0; j < X_SCALED; j++) {
			set_norm_matrix(drawing, j,i,0);
		}
	}
}

// set value at (x,y) in drawing
void set_norm_matrix(uint8_t drawing[X_SCALED][(Y_SCALED+7)/8], uint8_t x, uint8_t y, uint8_t val) {
	drawing[x][y/8] = (val ? drawing[x][y/8] | (1 << (y%8)) : drawing[x][y/8] & ~(1 << (y%8)));
}

// get value at (x,y) in drawing
uint8_t get_norm_matrix(uint8_t drawing[X_SCALED][(Y_SCALED+7)/8], uint8_t x, uint8_t y) {
	return !!(drawing[x][y/8] & (1 << (y%8)));
}

// Read 10-bit value from ADC
// Input argument:
// COORD_X maps to ADC0
// COORD_Y maps to ADC1
uint16_t read_touch_pad(coord_t coord_type) {
	
	set_PORTA_for_coord(coord_type);
	_delay_us(150); // give the pulldown resistor time to bring the value to 0
	// Start conversion
	ADCSRA |= (1<<ADSC);
	// Wait until conversion done (ADSC bit = 0 when done)
	while(ADCSRA & (1<<ADSC));
	// Read ADC data registers (low first)
	uint16_t low_byte = ADCL;
	uint16_t high_byte = ADCH;
	uint16_t val = (high_byte << 8) + low_byte;
	return val;
}

void set_PORTA_for_coord(coord_t coord_type) {
	switch(coord_type) {
	case COORD_X:
		// A0, A2 is an input, other 2 are outputs
		DDRA = 0b000001010;
		// Send 5V to A3, 0V to others. A0, A2 = Hi-Z
		PORTA = 0b00001000;
		ADC_PIN_SEL(ADC0);
		break;
	case COORD_Y:
		// A1, A3 is an input, other 2 are outputs.
		DDRA = 0b000000101;
		// Send 5V to A2, 0V to others. A1, A3 = Hi-Z
		PORTA = 0b00000100;
		ADC_PIN_SEL(ADC1);
		break;
	}
} 

void initialize(void) {
	// Enable ADC, set ADC prescaler to 128
	ADCSRA = (1<<ADEN) + 7;
	// Initialize AREF to be 5V
	ADC_VREF_SET(ADC_VREF_5);
	// Use ADC 0 to start out
	ADC_PIN_SEL(ADC0);

	// Other options:
	// Enable ADC conversion to be started by external source
	// ADCSRA |= (1<<ADATE);
	// ADC will start conversion upon Timer 0 Compare Match
	// ADCSRB = 3;
	// ADC will trigger interrupt upon completion of a conversion
	// ADCSRA |= (1<<ADIE);

	// Set up timer 0 for sample timing purposes
	// 1 ms ticks (assuming 16 MHz clock)
	TIMSK0 = 2;		// turn on timer 0 cmp match ISR 
	OCR0A = 250;  	// set the compare reg to 250 time ticks
	TCCR0A = 0b00000010; // turn on clear-on-match
	TCCR0B = 0b00000011;	// clock prescaler to 64

	// Init the UART -- uart_init() is in uart.c
    uart_init();
    stdout = stdin = stderr = &uart_str;
    fprintf(stdout,"Starting...\n\r");

	// LEDS: D4 green, D5 red
	DDRD |= 0b00110000;
	PORTD |= 0b00110000;

	sei();
}
