//Justin Churchill (jec324), Stephen Wu (shw46)
//ECE 4760 Lab 5
//FlashMemLoad

#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>
#include "constants.h"
#include "spiflash.h"
#include "uart.h"
#include "lcd_functions.h"

#define MATLAB_MULT_FACTOR 1000000

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

void initialize(void) {
	uart_init();

	stdout = stdin = stderr = &uart_str;
	SPI_MasterInit();
}

void forward_serial_to_flash(void) {
	char serial_buffer[FLASH_BUFFER_SIZE];

	unsigned long expected_longs_hid = (unsigned long)HIDDEN_NODE_CT*(X_SCALED*Y_SCALED+1);
	unsigned long expected_longs_outp = (unsigned long)OUTPUT_NODE_CT*(HIDDEN_NODE_CT+1);
	int page = PAGE_START_HID;
	int pg_st_out;
	// PAGE_START_OUT should end up being PAGE_START_HID+149 for our 180x108x26 NN
	int bytes_recvd = 0;
	long i;
	
	// Hidden layer weights bytes come first.
	for(i=0; i<expected_longs_hid; i++) {
		int r = -1;
		long val_recvd;
		floatbytes_t for_mem;
		
		while (r<=0) {
			r = fscanf(stdin, " %ld ", &val_recvd);
		}

		for_mem.fval = (float)val_recvd/MATLAB_MULT_FACTOR;

		PORTD = ((PORTD & LED_D4) ? (PORTD & ~(LED_D4)) : (PORTD | LED_D4));

		serial_buffer[bytes_recvd++] = (char)for_mem.cval[3];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[2];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[1];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[0];

		if (bytes_recvd == FLASH_BUFFER_SIZE) {
			bytes_recvd = 0;
			write_flash_page(serial_buffer, FLASH_BUFFER_SIZE, page++);	
		}
	}
	// Send last incomplete page of bytes received
	if(bytes_recvd > 0) {
		write_flash_page(serial_buffer, bytes_recvd, page++);
		bytes_recvd = 0;
	}

	pg_st_out = page;

	// Output layer weights bytes come second.
	for(i=0; i<expected_longs_outp; i++) {
		int r = -1;
		long val_recvd;
		floatbytes_t for_mem;
		
		while (r<=0) {
			r = fscanf(stdin, " %ld ", &val_recvd);
		}

		for_mem.fval = (float)val_recvd/MATLAB_MULT_FACTOR;

		PORTD = ((PORTD & LED_D4) ? (PORTD & ~(LED_D4)) : (PORTD | LED_D4));

		serial_buffer[bytes_recvd++] = (char)for_mem.cval[3];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[2];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[1];
		serial_buffer[bytes_recvd++] = (char)for_mem.cval[0];

		if (bytes_recvd == FLASH_BUFFER_SIZE) {
			bytes_recvd = 0; // Toggle red
			write_flash_page(serial_buffer, FLASH_BUFFER_SIZE, page++);
		}
	}
	// Send last incomplete page of bytes received
	if(bytes_recvd > 0) {
		write_flash_page(serial_buffer, bytes_recvd, page++);
	}
	
	LCD_clear();
	sprintf(lcd_buffer,"PG hid: %d-%d", PAGE_START_HID, pg_st_out-1);	                 
	LCDGotoXY(0, 0);
	LCDstring(lcd_buffer, strlen(lcd_buffer));
	sprintf(lcd_buffer,"PG out: %d-%d", pg_st_out, page-1);	                 
	LCDGotoXY(0, 1);
	LCDstring(lcd_buffer, strlen(lcd_buffer));
}


int main(void) {
	initialize();
	
	DDRD |= 0b00110000;
	PORTD &= ~(LED_D4);
	PORTD &= ~(LED_D5);
	forward_serial_to_flash();
	PORTD |= (LED_D5);
	PORTD |= (LED_D4);
	while(1);
}

