// Justin Churchill (jec324) Stephen Wu (shw46)
// ECE 4760 Lab 5
// NNClassifier.c

#include "NNClassifier.h"
#include "normdrawing.h"
#include "drawing.h"
#include "spiflash.h"
#include <stdio.h>
#include <stdlib.h>

// Calculate output at each hidden node, return as array.
// Note: with the way the matlab code is written, for a given hidden node,
//       the weight at index x*Y_SCALED+y maps to point (x,y) on the drawing.
void hidden_output(float out[HIDDEN_NODE_CT],
                   uint8_t input[X_SCALED][(Y_SCALED+7)/8]) {
    int hid, inpx, inpy;
    float sum;

	flash_read_init(PAGE_START_HID);

    for(hid=0; hid<HIDDEN_NODE_CT; hid++) {
        sum = 0;
		//fprintf(stdout, "Hid node %d\n", hid+1);
        // Calculate output at hidden node hid
        for(inpx=0; inpx<X_SCALED; inpx++) {
            for(inpy=0; inpy<Y_SCALED; inpy++) {
				float wt= flash_read_next_float();
                if(get_norm_matrix(input, inpx, inpy)) {
                    sum += wt;
					if(hid==0) {
						char val[10];
						dtostrf(wt, 6, 6, val);
						//fprintf(stdout, "HID(%2d,%2d): %s\n", inpy, inpx, val);
					}
                }
            }
        }
		// The last element is the bias
		sum += flash_read_next_float();
        // Sigmoid the sum
        sum = 1/(1+exp(-1*sum));
        out[hid] = sum;
    }

	flash_read_complete();
}

// Calculate output at each output node, return as array.
void output_output(float out[OUTPUT_NODE_CT],
                   float hidden_out[HIDDEN_NODE_CT]) {
    int outn, hid;
    float sum;

	flash_read_init(PAGE_START_OUT);

    for(outn=0; outn<OUTPUT_NODE_CT; outn++) {
        sum = 0;
		//fprintf(stdout, "Out node %d\n", outn+1);
        // Calculate output at output node outn
        for(hid=0; hid<HIDDEN_NODE_CT; hid++) {
			float wt = flash_read_next_float();
            sum += hidden_out[hid] * wt;
			if(outn==0) {
				char val[10];
				dtostrf(wt, 6, 6, val);
				//fprintf(stdout, "OUT(%3d): %s\n", hid+1, val);
            }
        }
		// The last element is the bias
		sum += flash_read_next_float();
        // Sigmoid the sum (no actual need for this, can be removed for speed)
        sum = 1/(1+exp(-1*sum));
        out[outn] = sum;
    }

	flash_read_complete();
}

char output_choice(float out[OUTPUT_NODE_CT]) {
    int node, ind_max = 0;
    for(node=0; node<OUTPUT_NODE_CT; node++) {
        if(out[ind_max] < out[node]) {
            ind_max = node;
        }
    }
    return 'A' + ind_max;
}


char NNclassify(uint8_t input[X_SCALED][(Y_SCALED+7)/8]) {
    float hidden_out[HIDDEN_NODE_CT];
    float output_out[OUTPUT_NODE_CT];

    // Calculate output at each hidden node.
    hidden_output(hidden_out, input);
    // Calculate output at each output node.
    output_output(output_out, hidden_out);
	// Find and return letter with highest score.
    return output_choice(output_out);
}


//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

#define THRESHOLD_SZ 4

typedef enum direction {
    NONE,
    NORTH,
    NEAST,
    EAST,
    SEAST,
    SOUTH,
    SWEST,
    WEST,
    NWEST
} dir_t;

// Prototypes
dir_t xy_2_dir(int, int);
char get(uint8_t input[X_RES][Y_RES/8], int, int);
int connected_mask(char mask[3][3], uint8_t input[X_RES][Y_RES/8], int, int, int);
int connected(uint8_t input[X_RES][Y_RES/8], dir_t, int, int, int, char br_checked[3][3]);

// Get direction associated with x={-1,0,1} and y={-1,0,1}
dir_t xy_2_dir(int dx, int dy) {
    if (dx==-1 && dy==-1) { return NWEST; }
    else if (dx==-1 && dy==0) { return WEST; }
    else if (dx==-1 && dy==1) { return SWEST; }
    else if (dx==0 && dy==-1) { return NORTH; }
    else if (dx==0 && dy==0) { return NONE; }
    else if (dx==0 && dy==1) { return SOUTH; }
    else if (dx==1 && dy==-1) { return NEAST; }
    else if (dx==1 && dy==0) { return EAST; }
    else if (dx==1 && dy==1) { return SEAST; }
	else { return NONE; } // Error
}

// Access the input matrix at point (x,y). If an out-of-bounds element
// is attempted to be accessed, return 0.
char get(uint8_t input[X_RES][Y_RES/8], int x, int y) {
    if(x<0 || x >= X_RES || y<0 || y >= Y_RES) {
        return 0;
    }
    return get_matrix(input, x, y);
}

// Helper function for cleanup recursion.
int connected_mask(char mask[3][3],
                   uint8_t input[X_RES][Y_RES/8],
                   int count_req,
                   int x, int y) {
    int dx, dy;
	int need_to_see;
	char tmp[3][3];
	char br_checked[3][3] = {{0,0,0},{0,0,0},{0,0,0}};

    for(dy=-1; dy<=1; dy++) {
        for(dx=-1; dx<=1; dx++) {
            tmp[1+dy][1+dx] = mask[1+dy][1+dx] && get(input,x+dx,y+dy);
            count_req -= tmp[1+dy][1+dx];
        }
    }

	need_to_see = count_req;

    if(need_to_see <= 0) {
        return 0;
    }
    else {
        for(dy=-1; dy<=1; dy++) {
            for(dx=-1; dx<=1; dx++) {
                dir_t d = xy_2_dir(dx, dy);
                if(tmp[1+dy][1+dx]) {
					br_checked[1+dy][1+dx] = 1; 
                    need_to_see -= (need_to_see - connected(input, d, count_req, x+dx, y+dy, br_checked));
					if(need_to_see <= 0) {
                        return 0;
					}
                }
            }
        }
    }
    return need_to_see;
}

// Clean up recursive method.
int connected(uint8_t input[X_RES][Y_RES/8], dir_t direction, int count_req, int x, int y, char br_checked[3][3]) {

    if(!get_matrix(input, x, y)) {
        return count_req;
    }

    if(direction == NONE) {
        // Assume br_checked = 0 everywhere in this case
        char mask[3][3] = {
            {1,1,1},
            {1,0,1},
            {1,1,1}
        };
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == NORTH) {
        char mask[3][3] = {
            {1,1,1},
            {0,0,0},
            {0,0,0}
        };
        if(br_checked[0][0]) {
            mask[0][0] = 0;
            mask[0][1] = 0;
        }
        if(br_checked[0][2]) {
            mask[0][1] = 0;
            mask[0][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == NEAST) {
        char mask[3][3] = {
            {1,1,1},
            {0,0,1},
            {0,0,1}
        };
        if(br_checked[0][1]) {
            mask[0][0] = 0;
            mask[0][1] = 0;
        }
        if(br_checked[1][2]) {
            mask[1][2] = 0;
            mask[2][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == EAST) {
        char mask[3][3] = {
            {0,0,1},
            {0,0,1},
            {0,0,1}
        };
        if(br_checked[0][2]) {
            mask[0][2] = 0;
            mask[1][2] = 0;
        }
        if(br_checked[2][2]) {
            mask[1][2] = 0;
            mask[2][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == SEAST) {
        char mask[3][3] = {
            {0,0,1},
            {0,0,1},
            {1,1,1}
        };
        if(br_checked[1][2]) {
            mask[0][2] = 0;
            mask[1][2] = 0;
        }
        if(br_checked[2][1]) {
            mask[2][0] = 0;
            mask[2][1] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == SOUTH) {
        char mask[3][3] = {
            {0,0,0},
            {0,0,0},
            {1,1,1}
        };
        if(br_checked[2][0]) {
            mask[2][0] = 0;
            mask[2][1] = 0;
        }
        if(br_checked[2][2]) {
            mask[2][1] = 0;
            mask[2][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == SWEST) {
        char mask[3][3] = {
            {1,0,0},
            {1,0,0},
            {1,1,1}
        };
        if(br_checked[1][0]) {
            mask[0][0] = 0;
            mask[1][0] = 0;
        }
        if(br_checked[2][1]) {
            mask[2][1] = 0;
            mask[2][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == WEST) {
        char mask[3][3] = {
            {1,0,0},
            {1,0,0},
            {1,0,0}
        };
        if(br_checked[0][0]) {
            mask[0][0] = 0;
            mask[1][0] = 0;
        }
        if(br_checked[2][0]) {
            mask[1][0] = 0;
            mask[2][0] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else if (direction == NWEST) {
        char mask[3][3] = {
            {1,1,1},
            {1,0,0},
            {1,0,0}
        };
        if(br_checked[1][0]) {
            mask[1][0] = 0;
            mask[2][0] = 0;
        }
        if(br_checked[0][1]) {
            mask[0][1] = 0;
            mask[0][2] = 0;
        }
        return connected_mask(mask, input, count_req, x, y);
    } else {
        return count_req;
	}
}

// Should this pixel be kept? (Is it part of an 8-connected chunk that
// is greater than the THRESHOLD_SZ)
int rid_pixel(uint8_t input[X_RES][Y_RES/8], int x, int y) {
	char br_ch[3][3] = {{0,0,0},{0,0,0},{0,0,0}};
    return connected(input, NONE, THRESHOLD_SZ, x, y, br_ch) > 0;
}

// Clean the entire input drawing.
void clean_input(uint8_t input[X_RES][Y_RES/8], uint8_t* x_min_found, uint8_t* x_max_found, uint8_t* y_min_found, uint8_t* y_max_found) {
    int x,y;
	*x_min_found = X_RES;
	*y_min_found = Y_RES;
	*x_max_found = 0;
	*y_max_found = 0;
    for(y=0;y<Y_RES;y++) {
        for(x=0;x<X_RES;x++) {
			if(get_matrix(input, x, y)) {
	            if(rid_pixel(input, x, y)) {
	                set_matrix(input, x, y, 0);
	            }
				else {
					*x_min_found = (x < *x_min_found ? x : *x_min_found);
					*y_min_found = (y < *y_min_found ? y : *y_min_found);
					*x_max_found = (x > *x_max_found ? x : *x_max_found);
					*y_max_found = (y > *y_max_found ? y : *y_max_found);
				}
			}
        }
    }
}

