//Justin Churchill (jec324), Stephen Wu (shw46)
//ECE 4760 Lab 5
//Touchpad Letter Recognition

// HARDWARE SETUP:
// PIN A0 : measures x coord. Goes to y1 on board.
// PIN A1 : measures y coord. Goes to x2 on board.
// PIN A2 : needs to be output to set 0 or 5 V. Goes to y2 on board.
// PIN A3 : needs to be output to set 0 or 5 V. Goes to x1 on board.
// 10k Resistor between A0 and A1.

// Measuring x: x2(A1) low, x1(A3) high. Sample y1(A0).
// Measuring y: y1(A0) low, y2(A2) high. Sample x2(A1).

// PIN A4 : backspace pushbutton
// PIN A5 : space pushbutton
// PIN A6 : clear pushbutton
// PIN A7 : Switch for POV mode on/off

// PIN D5 : Red LED
// PIN D4 : Green LED

// PIN B4-B7 : SPI

// PORT C : LCD Display

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>
#include <inttypes.h>
#include "uart.h"
#include "transmit.h"
#include "letters.h"

#include "constants.h"
#include "drawing.h"
#include "normdrawing.h"
#include "NNClassifier.h"
#include "lcd_functions.h"
#include "spiflash.h"

//FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

typedef enum coord_t {
	COORD_X,
	COORD_Y
} coord_t;

// Prototypes
void initialize(void);

uint16_t read_touch_pad(coord_t);
void set_PORTA_for_coord(coord_t);
int16_t sample2pt(coord_t, uint16_t);


// Global vars
uint16_t sample_time = TOUCHPAD_SAMPLE_PERIOD;
volatile uint8_t ADC_get_sample = false;
volatile uint8_t sample_on = false;

volatile uint8_t button_pressed = false;
volatile uint16_t button_time = 0; // Time until next acceptable button input
volatile uint16_t button_update_time = BUTTON_UPDATE_PERIOD;

uint8_t drawing[X_RES][Y_RES/8];
uint8_t norm_drawing[X_SCALED][(Y_SCALED+7)/8];

//////////////////////////////////////////////////////////////////////////

int main(void) {
	initialize();

	sample_on = true;
	uint16_t counter = 0;
	uint8_t started = false;
	for(;;) {
		if (button_update_time == 0 && button_pressed == false) {
			if (~PINA & 0b00010000) {
				//fprintf(stdout, "Backspace.\n");
				LCD_backspace();
				button_pressed = true;
			}
			if (~PINA & 0b00100000) {
				//fprintf(stdout, "Space.\n");
				LCD_space();
				button_pressed = true;
			}
			if (~PINA & 0b01000000) {
				//fprintf(stdout, "Clear.\n");
				LCD_clear();
				button_pressed = true;
			}
			POV_mode = ((~PINA & 0b10000000) ? true : false);
		}
		
		PORTD &= ~LED_D5; // Red off
		PORTD |= LED_D4;  // Green on		

		if(ADC_get_sample) {
			uint16_t x_samp, y_samp;
			int16_t x_conv, y_conv;
			ADC_get_sample = false;
			x_samp = 1024 - read_touch_pad(COORD_X); // it takes the sample or it gets the hose again
			y_samp = 1024 - read_touch_pad(COORD_Y); // IT TAKES THE SAMPLE OR IT GETS THE HOSE AGAIN
			x_conv= sample2pt(COORD_X, x_samp);
			y_conv= sample2pt(COORD_Y, y_samp);
			//fprintf(stdout, "Raw Data  (%d , %d)\n\r", x_samp, y_samp);
			//fprintf(stdout, "Converted (%d , %d)\n\r", x_conv, y_conv);
			
			if (x_conv >= 0 && y_conv >= 0) {
				set_matrix(drawing, x_conv, y_conv, 1);
				started = true;
				counter = 0;
			}

			if (started == true && (x_conv == -1 || y_conv == -1)) {
				counter++;
				if (counter == PAUSE_TIME) {
					uint8_t xb_min, xb_max, yb_min, yb_max;
					char drawn;
					PORTD &= ~LED_D4; // Green off
					PORTD |= LED_D5;  // Red on
					
					//print_drawing(drawing);
					clean_input(drawing, &xb_min, &xb_max, &yb_min, &yb_max);
					//print_drawing(drawing);

					change_size(drawing, norm_drawing, xb_min, xb_max, yb_min, yb_max);

					//print_norm_drawing(norm_drawing);

					drawn = NNclassify(norm_drawing);
					//fprintf(stdout, "You drew: %c\n", drawn);
					print_char_to_LCD(drawn, LCD_x, LCD_y);

					if(LCD_x < 14 && POV_mode == true) {
						print_char_to_POV(drawn, LCD_x, get_random_color());
						//send_drawing(norm_drawing, RED);
					}
					
					LCD_forward();
					//LCD_space();

					// FOR TRANSFERRING TO MATLAB
					//fprintf(stdout, "N\n");
					//print_drawing(drawing);
					//fprintf(stdout, "E\n");

					clear_drawing(drawing);
					clear_norm_drawing(norm_drawing);


					started= false;
				}
			}
		}
	}
	return 0;
}

// Timer 0 compare ISR
ISR (TIMER0_COMPA_vect)
{      
	// Touchpad sample rate
	if (sample_time > 0) {
		sample_time--;
	}
	else {
		sample_time = TOUCHPAD_SAMPLE_PERIOD;
		ADC_get_sample = true;
	}
	
	// Pushbutton debouncing
	if (button_update_time > 0) {
		button_update_time--;
	} else {
		button_update_time = BUTTON_UPDATE_PERIOD;
	}
	if (button_pressed == true && button_time > 0) {
		button_time--;
	}
	else { //button_time == 0
		button_time = BUTTON_DELAY_PERIOD;
		button_pressed = false;
	}
}



// converts a 10-bit ADC sample to a value between 0 and X_RES or Y_RES
// returns -1 if the stylus is not touching the screen
int16_t sample2pt(coord_t coord, uint16_t sample) {
	double norm;
	if (coord == COORD_X) {
		norm = (double)sample - X_MIN_1024;
		return (sample < X_MIN_1024 || sample > X_MAX_1024 ? -1 : (int16_t)(norm / (X_MAX_1024 - X_MIN_1024) * X_RES));
	}
	else if (coord == COORD_Y) {
		norm = (double)sample - Y_MIN_1024;
		return (sample < Y_MIN_1024 || sample > Y_MAX_1024 ? -1 : (int16_t)(norm / (Y_MAX_1024 - Y_MIN_1024) * Y_RES));
	}
	else
		return -2; //shouldn't get this
}

// Read 10-bit value from ADC
// Input argument:
// COORD_X maps to ADC0
// COORD_Y maps to ADC1
uint16_t read_touch_pad(coord_t coord_type) {
	
	set_PORTA_for_coord(coord_type);
	_delay_us(150); // give the pulldown resistor time to bring the value to 0
	// Start conversion
	ADCSRA |= (1<<ADSC);
	// Wait until conversion done (ADSC bit = 0 when done)
	while(ADCSRA & (1<<ADSC));
	// Read ADC data registers (low first)
	uint16_t low_byte = ADCL;
	uint16_t high_byte = ADCH;
	uint16_t val = (high_byte << 8) + low_byte;
	return val;
}

void set_PORTA_for_coord(coord_t coord_type) {
	switch(coord_type) {
	case COORD_X:
		// A0, A2 is an input, other 2 are outputs
		DDRA = 0b000001010;
		// Send 5V to A3, 0V to others. A0, A2 = Hi-Z
		PORTA = 0b00001000;
		ADC_PIN_SEL(ADC0);
		break;
	case COORD_Y:
		// A1, A3 is an input, other 2 are outputs.
		DDRA = 0b000000101;
		// Send 5V to A2, 0V to others. A1, A3 = Hi-Z
		PORTA = 0b00000100;
		ADC_PIN_SEL(ADC1);
		break;
	}
} 


void initialize(void) {
	// Enable ADC, set ADC prescaler to 128
	ADCSRA = (1<<ADEN) + 7;
	// Initialize AREF to be 5V
	ADC_VREF_SET(ADC_VREF_5);
	// Use ADC 0 to start out
	ADC_PIN_SEL(ADC0);

	// Set up timer 0 for sample timing purposes
	// 1/2 ms ticks (assuming 16 MHz clock)
	TIMSK0 = 2;		// turn on timer 0 cmp match ISR 
	OCR0A = 125;  	// set the compare reg to 250 time ticks
	TCCR0A = 0b00000010; // turn on clear-on-match
	TCCR0B = 0b00000011;	// clock prescaler to 64
	
	// Init the UART -- uart_init() is in uart.c
	//uart_init();
	//stdout = stdin = stderr = &uart_str;
	//fprintf(stdout,"ECE4760 jec324 shw46\n");

	// LEDS: D4 green, D5 red
	DDRD |= LED_D5 + LED_D4;
	PORTD |= LED_D5;

	// Enable communication with flash
	SPI_MasterInit();

	init_radio_uart();

	// setup LCD for PORTC operation
	init_LCD();

	sei();
}


