//Justin Churchill (jec324), Stephen Wu (shw46)
//ECE 4760 Lab 5
//drawing.c

// Allows for manipulation of an X_RES-by-Y_RES bitmap
// Rather than each pixel as an 8-bit value, the pixels
// are 0/1 values and one pixel is 1 bit. (Saves memory 8x!)

#include "drawing.h"
#include "normdrawing.h"
#include <stdio.h>

// clear the drawing
void clear_drawing(uint8_t drawing[X_RES][Y_RES/8]) {
	int i, j;
	for (i= 0; i < Y_RES; i++) {
		for (j= 0; j < X_RES; j++) {
			set_matrix(drawing, j,i,0);
		}
	}
}

// set value at (x,y) in drawing
void set_matrix(uint8_t drawing[X_RES][Y_RES/8], uint8_t x, uint8_t y, uint8_t val) {
	drawing[x][y/8] = (val ? drawing[x][y/8] | (1 << (y%8)) : drawing[x][y/8] & ~(1 << (y%8)));
}

// get value at (x,y) in drawing
uint8_t get_matrix(uint8_t drawing[X_RES][Y_RES/8], uint8_t x, uint8_t y) {
	return !!(drawing[x][y/8] & (1 << (y%8)));
}

// print the drawing
void print_drawing(uint8_t drawing[X_RES][Y_RES/8]) {
	int i, j;
	for (i= 0; i < Y_RES; i++) {
		for (j= 0; j < X_RES; j++) {
			fprintf(stdout, "%c", (get_matrix(drawing,j,i) == 1 ? '0' : ' '));
			//fprintf(stdout, "%c", (get_matrix(drawing,j,i) == 1 ? '1' : '0'));
		}
		fprintf(stdout, "\n");

	}
}

// Reduces a portion of the drawing to an X_SCALED-by-Y_SCALED resolution picture.
// Large drawing is in input, result is put into output.
// The last 4 arguments define the bounds on the box defining the portion of the drawing to scale.
void change_size(uint8_t input[X_RES][Y_RES/8], uint8_t output[X_SCALED][(Y_SCALED+7)/8], uint8_t x_begin, uint8_t x_end, uint8_t y_begin, uint8_t y_end) {
	uint8_t x, y, xb, yb, x_small, y_small;
	double conv_x, conv_y;
	x_small = x_end - x_begin + 1;
	y_small = y_end - y_begin + 1;
	conv_x = ((double)x_small) / X_SCALED;
	conv_y = ((double)y_small) / Y_SCALED;
	for (y = 0; y < Y_SCALED; y++) {
		for (x = 0; x < X_SCALED; x++) {
			int x_box_min = (int)(x*conv_x);
			int x_box_max = (int)((x+1)*conv_x-1);
			int y_box_min = (int)(y*conv_y);
			int y_box_max = (int)((y+1)*conv_y-1);
			int sum_pix = 0;
			for(yb = y_box_min; yb<= y_box_max; yb++) {
				for(xb = x_box_min; xb<= x_box_max; xb++) {
					sum_pix += get_matrix(input, x_begin+xb, y_begin+yb);
				}
			}
			
			if (((double)sum_pix)/((x_box_max-x_box_min+1)*(y_box_max-y_box_min+1)) > PIX_DENSITY_THRESHOLD) {
				set_norm_matrix(output, x, y, 1);
			}
		}
	}
}
