// Justin Churchill (jec324) Stephen Wu (shw46)
// ECE 4760 Lab 5
// spiflash.c

#include "spiflash.h"
#include "constants.h"
#include <avr/io.h>
#include <stdio.h>
#include <util/delay.h>

char junk=0xb0;

// send CS low
void SPI_CHIPSELECT(void) {
	PORTB &= ~(1<<DDB4);
}

// send CS high
void SPI_CHIPDESELECT(void) {
	PORTB |= (1<<DDB4);
	_delay_us(1);
}

// Initialize the microcontroller as a master.
void SPI_MasterInit(void) {
	// Set MOSI (B5) and SCK (B7) and SS (B4) output, MISO (B6) input
	DDRB &= ~(1<<DDB6);
	DDRB |= (1<<DDB3)|(1<<DDB4)|(1<<DDB5)|(1<<DDB7);
	// Enable SPI, Master, set clock rate fck/2
	// NOTE! SPI SCK NEEDS TO BE fck/64 WHEN WRITING TO FLASH FROM MATLAB!
	SPCR = (1<<SPE)|(1<<MSTR);
	SPSR = (1<<SPI2X);
	//Hold reset pin (B3) low for a moment to reset flash state machine
	SPI_CHIPDESELECT();
	PORTB &= ~(1<<3);
	_delay_ms(100);
	PORTB |= (1<<3);
}

// Perform a single transaction over SPI
// Functions as both a write and a read
char SPI_MasterTransmit(char cData) {
	// Start transmission
	SPDR = cData;
	// Wait for transmission complete
	while(!(SPSR & (1<<SPIF)));
	return SPDR;
}

// Retrieve the status register from Flash
char read_flash_SR(void) {
	char stat_byte;
    SPI_CHIPSELECT();
    SPI_MasterTransmit(0xD7); // opcode D7 to read status
	SPI_MasterTransmit(0x00); // don't care byte
	stat_byte = SPDR;
    SPI_CHIPDESELECT();
	return stat_byte;
}

// Is the Flash ready to perform a new r/w operation?
char flash_rdy(void) {
	return !!(read_flash_SR() & (1<<7));
}

// Initialize a session of reading contiguous values from Flash
// memory, starting at page page_num and byte 0.
void flash_read_init(int page_num) {
    // op-code 03H (33HMz max)
    // 3 byte address sequence:
    // 1 don't-care bit
    // 13 bits (PA12 - PA0) specify which page
    // 10 bits (BA9 - BA0) specify the starting byte (always 0)
    
    char opc = 0x03;

	char msg[4] = {
        opc,                  // opcode
        (char)(page_num>>6),  // upper 7 bits of page num
		(char)(page_num<<2),  // lower 6 bits of page num, upper 2 bits of byte num
		(char)(0x00)          // lower 8 bits of byte num
    };
	
	while(!flash_rdy()); // Wait until flash done with past write operations
    
	PORTD |= (LED_D5);

    SPI_CHIPSELECT();
	// Send read instruction to flash
	for(int i=0; i<4; i++) {
		SPI_MasterTransmit(msg[i]);
	}
}

// Close a session of reading contiguous values from Flash
void flash_read_complete(void) {
    SPI_CHIPDESELECT();
}

// May only be called after flash_read_init has been
// called to specify the to begin reading from
//
// After done using this function, must call
// flash_read_complete before calling flash_read_init again!
char flash_read_next_byte() {
	return SPI_MasterTransmit(junk); // Do this just to spike SCK
}

// Writes the first byte_ct bytes in data_buf to the page page_num
// at byte 0 in Flash memory. Erases the page entirely before writing
// new values.
// LEDs also flash on port D to show that a write op is occurring.
// byte_ct MUST only range from 0-528!
void write_flash_page(char* data_buf, int byte_ct, int page_num) {
	// Op-code 82H (using buffer 1)
	// The 3 address bytes:
	// 1 dont care bit
	// 13 page address bits, (PA12 - PA0) 
	// 10 buffer address bits (BFA9 - BFA0) (we always choose 0)

	char msg[4] = {
		(char)(0x82),        // opcode
		(char)(page_num>>6), // upper 7 bits of page num
		(char)(page_num<<2), // lower 6 bits of page num, upper 2 bits of byte num
		(char)(0x00)         // lower 8 bits of byte num
	};

	while(!flash_rdy()); // Wait until flash done with past write operations

	PORTD |= (LED_D5);

	SPI_CHIPSELECT();
	// Send message to flash
	for(int i=0; i<4; i++) {
		SPI_MasterTransmit(msg[i]);
	}
	// Send data to flash
	for(int i=0;i<byte_ct;i++) {
		SPI_MasterTransmit(data_buf[i]);
	}
	SPI_CHIPDESELECT();

	PORTD &= ~(LED_D5);
}

// Read next 4 bytes from flash memory and convert to a single float number
// May only be called after flash_read_init has been called
float flash_read_next_float(void) {
	floatbytes_t memval;
	memval.cval[3] = flash_read_next_byte();
	memval.cval[2] = flash_read_next_byte();
	memval.cval[1] = flash_read_next_byte();
	memval.cval[0] = flash_read_next_byte();
	//fprintf(stdout, "%x %x %x %x\n", memval.cval[3],memval.cval[2],memval.cval[1],memval.cval[0]);
	return memval.fval;
}
