//Justin Churchill (jec324), Stephen Wu (shw46)
//ECE 4760 Lab 5
//transmit.c

// library for interfacing with the wireless comm board
// don't try to use this and PC UART at the same time

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <util/delay.h>
#include <stdio.h>
#include "uart.h"

/* system definitions*/

#ifndef F_CPU                       // defines system clock frequency
#define F_CPU        16000000
#endif

#define BAUDRATE  2400                         // define the BAUDRATE
#define UBRRVAL   ((F_CPU/(BAUDRATE*16UL))-1)
#define SYNC      0xAA                         // synchro signal
#define ANTISYNC  0x7F                         // synchro signal
#define ADDR      0xbb                         // address in memory

// bit handling macros for i/o registers
#define READ(U, N) ((U) >> (N) & 1u)
#define SET(U, N) ((void)((U) |= 1u << (N)))
#define CLR(U, N) ((void)((U) &= ~(1u << (N))))
#define FLIP(U, N) ((void)((U) ^= 1u << (N)))


//USART SEND
void USART_SENDBYTE(unsigned char SendByte) { 
	while(!(UCSR0A & (1<<UDRE0)));
	UDR0 = SendByte;
}

// Send Packet
void radio_send_packet(uint8_t row_addr, uint8_t col_addr, uint16_t msg) {

	USART_SENDBYTE(SYNC);
	USART_SENDBYTE(ADDR);

	//Column Address
	USART_SENDBYTE(col_addr);

	USART_SENDBYTE((uint8_t)(msg & 0x000F));
	USART_SENDBYTE((uint8_t)((msg & 0x00F0)>>4));
	USART_SENDBYTE((uint8_t)((msg & 0x0F00)>>8));
	USART_SENDBYTE(row_addr);

	USART_SENDBYTE(ANTISYNC);
}

// initialize registers for radio uart comm
void init_radio_uart(void) {
	UBRR0H = (uint8_t) (UBRRVAL>>8);  	// high byte
	UBRR0L = (uint8_t) UBRRVAL;   		// low byte        
	UCSR0B = (1<<TXEN0);  				// enable transmitter	
	// data frame format, asynch mode, no parity, 1 stop bit, 8 bit size
    UCSR0C = (0<<UMSEL00)|(0<<UMSEL01)|(0<<UPM00)|(0<<UPM01)|(0<<USBS0)|(3<<UCSZ00);    
}	
